# -*- coding: utf-8 -*-
"""
    flask_konch
    ~~~~~~~~~~~

    An improved shell commmand for the Flask CLI.

    :copyright: (c) 2016 by Steven Loria
    :license: MIT, see LICENSE for more details.
"""
__version__ = '1.1.0'
__author__ = 'Steven Loria'
__license__ = 'MIT'

__all__ = [
    'EXTENSION_NAME',
]

EXTENSION_NAME = 'flask-konch'



# -*- coding: utf-8 -*-
"""Entry point for the ``konch`` Flask CLI commmand."""
from flask.cli import with_appcontext
import click
import flask
import konch


DEFAULTS = dict(
    KONCH_FLASK_IMPORTS=True,
    KONCH_CONTEXT={},
    KONCH_SHELL='auto',
    KONCH_BANNER=None,
    KONCH_PROMPT=None,
    KONCH_OUTPUT=None,
    KONCH_CONTEXT_FORMAT=None,
    KONCH_IPY_AUTORELOAD=False,
    KONCH_IPY_EXTENSIONS=None,
    KONCH_PTPY_VI_MODE=False,
)


def get_flask_imports():
    ret = {}
    for name in [e for e in dir(flask) if not e.startswith('_')]:
        ret[name] = getattr(flask, name)
    return ret

@click.command()
@with_appcontext
def cli():
    """An improved shell command, based on konch."""
    from flask.globals import _app_ctx_stack
    app = _app_ctx_stack.top.app
    options = {
        key: app.config.get(key, DEFAULTS[key]) for key in DEFAULTS.keys()
    }
    base_context = {
        'app': app,
    }
    if options['KONCH_FLASK_IMPORTS']:
        base_context.update(get_flask_imports())

    context = dict(base_context)
    context.update(options['KONCH_CONTEXT'])

    def context_formatter(ctx):
        formatted_base = ', '.join(sorted(base_context.keys()))
        ret = '\n{FLASK}\n{base_context}\n'.format(
            FLASK=click.style('Flask:', bold=True),
            base_context=formatted_base
        )
        if options['KONCH_CONTEXT']:
            variables = ', '.join(sorted(options['KONCH_CONTEXT'].keys()))
            ret += '\n{ADDITIONAL}\n{variables}'.format(
                ADDITIONAL=click.style('Additional variables:', bold=True),
                variables=variables
            )
        return ret

    context_format = options['KONCH_CONTEXT_FORMAT'] or context_formatter
    konch.start(
        context=context,
        shell=options['KONCH_SHELL'],
        banner=options['KONCH_BANNER'],
        prompt=options['KONCH_PROMPT'],
        output=options['KONCH_OUTPUT'],
        ptpy_vi_mode=options['KONCH_PTPY_VI_MODE'],
        context_format=context_format,
        ipy_autoreload=options['KONCH_IPY_AUTORELOAD'],
    )



# -*- coding: utf-8 -*-
import re
from setuptools import setup, find_packages


REQUIRES = [
    'Flask>=0.11',
    'konch>=2.0.0',
]

def find_version(fname):
    '''Attempts to find the version number in the file names fname.
    Raises RuntimeError if not found.
    '''
    version = ''
    with open(fname, 'r') as fp:
        reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
        for line in fp:
            m = reg.match(line)
            if m:
                version = m.group(1)
                break
    if not version:
        raise RuntimeError('Cannot find version information')
    return version

__version__ = find_version('flask_konch/__init__.py')


def read(fname):
    with open(fname) as fp:
        content = fp.read()
    return content

setup(
    name='flask-konch',
    version=__version__,
    description='An improved shell command for the Flask CLI',
    long_description=read('README.rst'),
    author='Steven Loria',
    author_email='sloria1@gmail.com',
    url='https://github.com/sloria/flask-konch',
    packages=find_packages(exclude=("test*", )),
    package_dir={'flask-konch': 'flask-konch'},
    include_package_data=True,
    install_requires=REQUIRES,
    license=read("LICENSE"),
    zip_safe=False,
    keywords='flask-konch',
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    test_suite='tests',
    entry_points={
        'flask.commands': [
            'konch=flask_konch.cli:cli'
        ]
    },
)
